import time

def send_metrics(sender, metrics):
    """Serialize the given metrics dict (assumed to be key:value pairs without timestamps) to triplets in a 
    multi-line message, using a timestamp generated in this method. Once the message is formatted, send it to 
    the configured Slack channel.
    """
    now = int(time.time())

    lines = []
    for k in metrics.keys():
        v = metrics[k]
        lines.append(f"{k} {v} {now}")

    sender.send_raw("\n".join(lines))



import pika
import time

URL = 'url'
QUEUE = 'queue'

DEFAULT_QUEUE = 'mulay'

class AMQPConfig(object):
    def __init__(self, config_dict):
        self.url = config_dict[URL]
        self.queue = config.get(QUEUE) or DEFAULT_QUEUE

class Relay(object):
    def __init__(self, config_dict, sender):
        self.config = AMQPConfig(config_dict)
        self.sender = sender

    def on_message(self, body):
        for line in body.splitlines():
            self.sender.send_raw(line)

    def start(self):
        params = pika.URLParameters(self.config.url)
        self.conn = pika.BlockingConnection(params)
        self.chan = self.conn.channel()

        self.chan.queue_declare(queue=self.config.queue)
        self.chan.basic_consume(self.config.queue, lambda _c, _m, _p, body: self.on_message(body), auto_ack=True)
        self.chan.start_consuming()

    def stop(self):
        self.chan.stop_consuming()
        self.conn.close()

class Sender(object):
    def __init__(self, config):
        self.config = AMQPConfig(config_dict)

    def start(self):
        params = pika.URLParameters(self.config.url)
        self.conn = pika.BlockingConnection(params)
        self.chan = self.conn.channel()

    def stop(self):
        self.conn.close()

    def send_raw(self, body):
        self.chan.basic_publish(exchange='', routing_key=self.config.queue, body=body)




import socket

HOST='host'
PORT='port'

DEFAULT_PORT=2023

class CarbonConfig(object):
    def __init__(self, config_dict):
        self.host = config_dict[HOST]
        self.port = config_dict.get(PORT) or DEFAULT_PORT

class Sender(object):
    def __init__(self, config_dict):
        """Initialize a new Carbon sender that will use the plaintext protocol for sending metrics"""
        self.config = CarbonConfig(config_dict)

    def start(self):
        self.sock = socket.socket()
        conn_info = (self.config.host, self.config.port)
        try:
            print("Connecting to: %s:%d" % conn_info)
            self.sock.connect(conn_info)
        except socket.error:
            raise SystemExit("Couldn't connect to %s:%d, is carbon-cache.py running?" % conn_info)

    def stop(self):
        """If a socket is open for sending data at the time when the user kills this script, try to close it 
        gracefully.
        """
        if self.sock is not None:
            print("Attempting to close socket.")
            self.sock.close()
            self.sock = None

    def send_metrics(self, metrics):
        """Serialize the metrics dict (of the form k:v without timestamps) along with a generated timestamp into a
        multi-line payload. Then, send it over a new socket to the Carbon daemon.
        """
        for line in metrics:
            print("Sending:\n%s\n" % line)
            send_raw(line)

    def send_raw(self, line):
        try:
            self.sock.send((line + "\n").encode())
        except socket.error:
            self.stop()
            self.start()




#!/usr/bin/env python2

from setuptools import setup, find_packages
import sys

with open('README.rst', 'r') as f:
  long_description=f.read()

version='0.2.0'

setup(
    zip_safe=True,
    name='mulay',
    version=version,
    description="Relay / sender library for various bus / consumer types",
    long_description=long_description,
    classifiers=[
      "Development Status :: 3 - Alpha",
      "Intended Audience :: Developers",
      "License :: OSI Approved :: GNU General Public License (GPL)",
      "Programming Language :: Python :: 3",
      "Topic :: Utilities",
    ],
    keywords='graphite carbon metrics monitoring mqtt amqp slack',
    author='John Casey',
    author_email='jdcasey@commonjava.org',
    url='https://github.com/jdcasey/mulay',
    license='GPLv3+',
    packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
    install_requires=[
      'pika',
      'paho-mqtt',
      'slackclient',
      'click',
      'ruamel.yaml'
    ],
    test_suite="tests"
)




from ruamel.yaml import YAML

def load(config_path):
    """Load a Carbon-AMQP relay configuration from a YAML file (by default, use /etc/carbon-amqp.yml)"""
    data = {}

    with open(config_path) as f:
        data = YAML().load(f)

    return data





# import context
import paho.mqtt.client as mqtt

TOPIC='topic'
USER = 'user'
PASSWORD = 'password'
HOST = 'host'
PORT = 'port'

DEFAULT_PORT = 1883
DEFAULT_TOPIC = 'mulay'

class MQTTConfig(object):
    def __init__(self, config_dict):
        self.host = config_dict[HOST]
        self.port = config_dict.get(PORT) or DEFAULT_PORT
        self.topic = config_dict.get(TOPIC) or DEFAULT_TOPIC
        self.username = config_dict.get(USER) or ''
        self.password = config_dict.get(PASSWORD) or ''

class Relay(mqtt.Client):
    def __init__(self, config_dict, sender):
        super(Relay, self).__init__()
        self.config = MQTTConfig(config_dict)
        self.sender = sender
        self.connected = False
        self.subscribed = False
        if self.config.password is not None:
            self.username_pw_set(username=self.config.username,password=self.config.password)

    def on_connect(self, mqttc, obj, flags, rc):
        self.connected = True
        print(f"Connected: {str(rc)}")

    def on_subscribe(self, mqttc, obj, mid, granted_qos):
        self.subscribed = True
        print(f"Subscribed to: {str(mid)}")

    def on_log(self, mqttc, obj, level, string):
        print(f"MQTT:[{level}] {string}")

    def on_message(self, mqttc, obj, msg):
        # print(f"Message: {msg}")
        for line in msg.payload.splitlines():
            self.sender.send_raw(line)

    def start(self):
        print(f"Starting relay on: {self.config.host}:{self.config.port}")
        self.connect(self.config.host, self.config.port, 60)

        print(f"Subscribing to: {self.config.topic}")
        self.subscribe(self.config.topic, 0)
        self.loop_forever()

    def stop(self):
        if self.subscribed is True:
            self.unsubscribe(self.config.topic)

        if self.connected is True:
            self.disconnect()

class Sender(mqtt.Client):
    def __init__(self, config_dict):
        super(Sender, self).__init__()
        self.config = MQTTConfig(config_dict)
        self.connected = False
        if self.config.password is not None:
            self.username_pw_set(username=self.config.username,password=self.config.password)

    def __del__(self):
        super().__del__()

    def on_connect(self, mqttc, obj, flags, rc):
        self.connected = True
        print(f"Connected: {str(rc)}")

    def on_publish(self, mqttc, obj, mid):
        print(f"Published: {str(mid)}")

    def on_log(self, mqttc, obj, level, string):
        print(f"MQTT:[{level}] {string}")

    def start(self):
        self.connect(self.config.host, self.config.port, 60)
        self.loop_start()

    def stop(self):
        if self.connected is True:
            self.loop_stop()
            self.disconnect()

    def send_raw(self, text):
        message_info = self.publish(self.config.topic, text, qos=0)
        message_info.wait_for_publish()




class Sender(object):
    def __init__(self, config_dict):
        print("Console Sender Initialized")

    def start(self):
        print("Console Sender Started")

    def stop(self):
        print("Console Sender Stopped")

    def send_raw(self, text):
        print(text)



import slackclient as client
import time

def find_channel_id(slack, name):
    """Allow a user to configure this system using a user-friendly channel name.
    The network handlers will use this method to map that to a channel ID.
    """
    result = slack.api_call("channels.list", exclude_members=True)
    for channel in result['channels']:
        if channel['name'] == name:
            return channel['id']

        prev_names = channel.get('previous_names')
        if prev_names is not None and name in prev_names:
            return channel['id']

    return None

class SlackConfig(object):
    def __init__(self, config_dict):
        self.token = config_dict[TOKEN]
        self.channel = config_dict[CHANNEL]

class Relay(object):
    def __init__(self, config_dict, sender, last_message=0):
        """Initialize the Relay using a config dict and an optional last-message timestamp (in unix seconds).
        """
        self.config = SlackConfig(config_dict)
        self.slack = client.SlackClient(self.config.token)
        self.sender = sender

        self.channel_id = find_channel_id(self.slack, self.config.channel)
        if self.channel_id is not None:
            self.slack.api_call('channels.join', channel=self.channel_id)

        self.last_message = last_message

    def stop(self):
        print("Stopped Slack relay")

    def _get_messages(self):
        """Receive any available messages on the Slack channel, and update the last-message timestamp
        appropriately before returning the array of message objects.
        """
        result = self.slack.api_call('channels.history', channel=self.channel_id, oldest=self.last_message)
        self.last_message=int(time.time())

        messages = result.get('messages') or []
        messages = messages[::-1]

        return messages

    def _ack_messages(self, messages_ts):
        """Acknowledge that processing is complete for an array of messages (given by their 'ts' values in messages_ts).
        This acknowledgement happens by way of deleting the messages out of the channel to prevent duplicate processing.
        """
        if messages_ts is not None:
            for ts in messages_ts:
                self.slack.api_call('chat.delete', channel=self.channel_id, ts=ts)

    def start(self):
        """Receive any available messages from the Slack channel and assume they are metrics triplets (key, value, tstamp).
        Messages can be multi-line, with a triplet per line. For each message that we can parse in this way,
        add the message 'ts' value to an acks array, which will be used to delete those messages from the 
        Slack channel to reduce the possibility of duplicate processing.

        For each triplet we can parse, send a new metric data point over a socket that we open to the Carbon
        daemon for this purpose, using newline-delimited plaintext.
        """
        while True:
            messages = self._get_messages()
            if len(messages) > 0:
                acks = []
                metrics = []
                for m in messages:
                    lines = [l.rstrip() for l in m['text'].splitlines()]
                    for line in lines:
                        parts = line.split(' ')
                        if len(parts) < 3:
                            continue

                        metrics.append("%s %s %s\n" % (parts[0], parts[1], parts[2]))
                    acks.append(m['ts'])

                if len(acks) > 0:
                    for m in metrics:
                        self.sender.send_raw(m)
                    self._ack_messages(acks)

            time.sleep(10)


class Explorer(object):
    def __init__(self, config_dict, last_message=0):
        """Construct a new Slack channel message receiver using the given configuration dict and an
        optional last-message timestamp (unix seconds).
        """
        self.config = SlackConfig(config_dict)
        self.slack = client.SlackClient(self.config.token)

        self.channel_id = find_channel_id(self.slack, self.config.channel)
        if self.channel_id is not None:
            self.slack.api_call('channels.join', channel=self.channel_id)

    def list_channels(self):
        """List the available Slack channels."""
        result = self.slack.api_call("channels.list")
        return [channel for channel in result['channels']]

    def find_user(self, user_id):
        """Lookup the user information for the given user id. Things like username will be available this way.
        """
        result = self.slack.api_call('users.info', user=user_id, include_locale=False)
        return result.get('user')

class Sender(object):
    def __init__(self, config_dict):
        """Construct a new Slack channel message sender using the given configuration dict to specify a token and 
        channel name.
        """
        self.config = SlackConfig(config_dict)

    def start(self):
        self.slack = client.SlackClient(self.config.token)

        self.channel_id = find_channel_id(self.slack, self.config.channel)
        if self.channel_id is not None:
            self.slack.api_call('channels.join', channel=self.channel_id)

    def stop(self):
        print("Stopped Slack sender")

    def send_metrics(self, metrics):
        """Serialize the given metrics dict (assumed to be key:value pairs without timestamps) to triplets in a 
        multi-line message, using a timestamp generated in this method. Once the message is formatted, send it to 
        the configured Slack channel.
        """
        now = int(time.time())

        lines = []
        for k in metrics.keys():
            v = metrics[k]
            lines.append("%s %s %s" % (k, v, now))

        self.send_raw("\n".join(lines))

    def send_raw(self, message):
        """Send a simple string message to the configured Slack channel."""
        return self.slack.api_call('chat.postMessage', channel=self.channel_id, text=message)
